#import "LVRTCEnginePlugin.h"
#import <LinkV/LinkV.h>
#import "FlutterSurfaceViewFactory.h"

@interface LVRTCEnginePlugin ()<LVRTCEngineDelegate>

@property (nonatomic, strong) LVRTCEngine *engine;
@property (nonatomic, copy) FlutterResult loginRoomResult;
@property (nonatomic, strong) FlutterMethodChannel *channel;

@end

@implementation LVRTCEnginePlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"LVRTCEngine" binaryMessenger:registrar.messenger];
    LVRTCEnginePlugin *plugin = [LVRTCEnginePlugin new];
    plugin.channel = channel;
    [registrar addMethodCallDelegate:plugin channel:channel];
    
    // 注册我们创建的view，这里的withId需要和flutter侧相同
    [registrar registerViewFactory:[FlutterSurfaceViewFactory new] withId:@"surface"];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    if ([call.method isEqualToString:@"getPlatformVersion"]) {
        result([NSString stringWithFormat:@"iOS - %@", [UIDevice currentDevice].systemVersion]);
    } else if ([call.method isEqualToString:@"initSDK"]) {
        [self initSDK:call result:result];
    } else if ([call.method isEqualToString:@"unInitSDK"]) {
        [self unInitSDK:result];
    } else if ([call.method isEqualToString:@"loginRoom"]) {
        [self loginRoom:call result:result];
    } else if ([call.method isEqualToString:@"logoutRoom"]) {
        [self logoutRoom:result];
    } else if ([call.method isEqualToString:@"setPublishQualityMonitorCycle"]) {
        [self setPublishQualityMonitorCycle:call result:result];
    } else if ([call.method isEqualToString:@"startPublishing"]) {
        [self startPublishing:result];
    } else if ([call.method isEqualToString:@"getSdkVersion"]) {
        result(LVRTCEngine.build_version);
    } else if ([call.method isEqualToString:@"startCapture"]) {
        [self startPublishing:result];
    } else if ([call.method isEqualToString:@"stopCapture"]) {
        [self stopCapture:result];
    } else if ([call.method isEqualToString:@"stopPublishStream"]) {
        [self stopPublishStream:result];
    } else if ([call.method isEqualToString:@"startPlayingStream"]) {
        [self startPlayingStream:call result:result];
    } else if ([call.method isEqualToString:@"stopPlayingStream"]) {
        [self stopPlayingStream:call result:result];
    } else if ([call.method isEqualToString:@"setPlayQualityMonitor"]) {
         [self setPlayQualityMonitor:call result:result];
    } else if ([call.method isEqualToString:@"switchCamera"]) {
        [self switchCamera:call result:result];
    } else if ([call.method isEqualToString:@"setAVConfig"]) {
        [self setAVConfig:call result:result];
    }
}

#pragma mark - LinkV method
- (void)initSDK:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    if (!self.engine) {
        [LVRTCEngine setUseTestEnv:[dict[@"isTestEnv"] boolValue]];
        [LVRTCEngine setUseInternationalEnv:[dict[@"isInternationalEnv"] boolValue]];
        [LVRTCEngine initSdk];
        
        self.engine = [LVRTCEngine sharedInstance];
        [self.engine setAVConfig:[[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_360P]];
    }
    
    [self.engine auth:dict[@"AppId"] skStr:dict[@"AppSign"] userId:nil completion:^(LVErrorCode code) {
        result(@(code));
    }];
}

- (void)unInitSDK:(FlutterResult)result {
    if (self.engine) {
//        [LVRTCEngine unInitSDK];
        self.engine = nil;
    }
}

- (void)loginRoom:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    self.loginRoomResult = result;
    
    NSLog(@"[Wing] loginRoom start");
    [self.engine loginRoom:dict[@"userId"] roomId:dict[@"roomId"] isHost:[dict[@"isHost"] boolValue] isOnlyAudio:[dict[@"isAudioOnly"] boolValue] delegate:self];
}

- (void)logoutRoom:(FlutterResult)result {
    NSLog(@"[Wing] logoutRoom start");
    [self.engine logoutRoom:^(LVErrorCode code) {
        NSLog(@"[Wing] logoutRoom end code = %@", @(code));
        result(@(code));
    }];
}

- (void)setPublishQualityMonitorCycle:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    [LVRTCEngine setPublishQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)startPublishing:(FlutterResult)result {
    BOOL isSucc = [self.engine startPublishing];
    result(@(isSucc));
}

- (void)stopCapture:(FlutterResult)result {
    [self.engine stopCapture];
    result(@(YES));
}

- (void)stopPublishStream:(FlutterResult)result {
    [self.engine stopPublishing];
    result(@(YES));
}

- (void)startPlayingStream:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    NSLog(@"[Wing] startPlayingStream streamId = %@", dict[@"streamId"]);
    [self.engine startPlayingStream:dict[@"streamId"]];
}

- (void)stopPlayingStream:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    NSLog(@"[Wing] stopPlayingStream streamId = %@", dict[@"streamId"]);
    [self.engine stopPlayingStream:dict[@"streamId"]];
}

- (void)setPlayQualityMonitor:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    [LVRTCEngine setPlayQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)switchCamera:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    
    LVRTCCameraPosition position = [dict[@"cameraPosition"] intValue] == 0 ? LVRTCCameraPositionBack : LVRTCCameraPositionFront;
    [[LVRTCEngine sharedInstance] switchCamera:position];
    result(@(YES));
}

- (void)setAVConfig:(FlutterMethodCall *) call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = call.arguments;
    int configValue = [dict[@"config"] intValue];
    
    LVAVConfig *config;
    switch (configValue) {
        case 0:
            config = [[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_180P];
            break;
       case 1:
            config = [[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_240P];
            break;
        case 2:
            config = [[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_360P];
            break;
        case 3:
            config = [[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_540P];
            break;
        case 4:
            config = [[LVAVConfig alloc] initWithVideoProfile:LVRTCVideoProfile_720P];
            break;
        default:
            break;
    }
    
    if (config) {
        [[LVRTCEngine sharedInstance] setAVConfig:config];
    }
}

#pragma mark - LVRTCEngineDelegate
/// 加入房间成功的回调
/// @param code 加入房间是否成功错误码
/// @param users 用户列表，如果房间中有用户存在，该列表会自动生效
- (void)OnEnterRoomComplete:(LVErrorCode)code users:(nullable NSArray<LVUser *> *)users {
    
    NSLog(@"[Wing] login room end code = %@", @(code));
    NSMutableArray *userArrM = [NSMutableArray array];
    if (code == LVErrorCodeSuccess) {
        for (LVUser *user in users) {
            NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
            dictM[@"userId"] = user.userId;
            dictM[@"pullUrls"] = user.pullUrls;
            
            [userArrM addObject:dictM];
        }
    }
    
    if (!self.loginRoomResult) return;
    
    NSDictionary *dict = @{@"code": @(code), @"users": userArrM};
    self.loginRoomResult(dict);
    self.loginRoomResult = nil;
}

- (void)OnAddRemoter:(LVUser *)user {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"userId"] = user.userId;
    dictM[@"pullUrls"] = user.pullUrls;
    
    NSLog(@"[Wing] OnAddRemoter uid = %@", user.userId);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onAddRemoter" arguments:dictM];
    });
}

- (void)OnDeleteRemoter:(NSString *)uid {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"streamId"] = uid;
    
    NSLog(@"[Wing] OnDeleteRemoter uid = %@", uid);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onDeleteRemoter" arguments:dictM];
    });
}

- (void)OnKickOff:(NSInteger)reason roomId:(NSString *)roomId {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"roomId"] = roomId;
    dictM[@"reason"] = @(reason);
    
    NSLog(@"[Wing] OnDeleteRemoter uid = %@", roomId);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onKickOff" arguments:dictM];
    });
}

- (void)OnRoomDisconnected:(LVErrorCode)code {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"code"] = @(code);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onRoomDisconnect" arguments:dictM];
    });
}

- (void)OnRoomReconnected {
    dispatch_async(dispatch_get_main_queue(), ^{
       [self.channel invokeMethod:@"onRoomReconnect" arguments:nil];
    });
}

- (void)OnMixComplete:(LVErrorCode)code {
    BOOL isSucc = code == LVErrorCodeSuccess;
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"b"] = @(isSucc);
    dispatch_async(dispatch_get_main_queue(), ^{
       [self.channel invokeMethod:@"onMixComplete" arguments:dictM];
    });
}

- (void)OnPublishQualityUpdate:(LVVideoStatistic *)quality {
    NSLog(@"[Wing] OnPublishQualityUpdate fps = %@", @(quality.videoFps));
    
    NSDictionary *dict = @{@"videoSentPackets": @(quality.videoSentPackets), @"videosentKbytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videobitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight)};
    dispatch_async(dispatch_get_main_queue(), ^{
       [self.channel invokeMethod:@"onPublishQualityUpdate" arguments:dict];
    });
}

- (void)OnPublishStateUpdate:(LVErrorCode)code {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"state"] = @(code);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onPublishStateUpdate" arguments:dictM];
    });
}

- (void)OnPlayQualityUpate:(nonnull LVVideoStatistic *)quality userId:(nonnull NSString *)userId {
    NSLog(@"[Wing] onPlayQualityUpdate fps = %@ uid = %@", @(quality.videoFps), userId);
    
    NSDictionary *dict = @{@"videoSentPackets": @(quality.videoSentPackets), @"videosentKbytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videobitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight), @"uid": userId ?: @""};
    dispatch_async(dispatch_get_main_queue(), ^{
       [self.channel invokeMethod:@"onPlayQualityUpdate" arguments:dict];
    });
}

- (void)OnPlayStateUpdate:(LVErrorCode)code userId:(nonnull NSString *)userId {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"state"] = @(code);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onPlayStateUpdate" arguments:dictM];
    });
}

- (void)OnExitRoomComplete {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onExitRoomComplete" arguments:nil];
    });
}

- (void)OnRoomDisconnect:(LVErrorCode)code {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onRoomDisconnect" arguments:@{@"code": @(code)}];
    });
}

- (void)OnAudioData:(NSString *)uid audio_data:(const void *)audio_data bits_per_sample:(int)bits_per_sample sample_rate:(int)sample_rate number_of_channels:(size_t)number_of_channels number_of_frames:(size_t)number_of_frames {
    
}

- (void)OnAudioMixStream:(nonnull const int16_t *)data samples:(int)samples nchannel:(int)nchannel flag:(LVAudioRecordType)flag {
    
}


- (void)OnAudioVolumeUpdate:(nonnull NSArray<LVAudioVolume *> *)soundLevels {
    
}


- (void)OnCaptureSoundLevelUpdate:(nonnull LVAudioVolume *)captureSoundLevel {
    
}


- (void)OnPublishResolutionSizeChanged:(CGSize)newSize bps:(int)bps {
    
}


@end
